function [pts, tri, land] = getEarthFlat(N_unif, N_adap),
% function [pts, tri, land] = getEarthFlat(N_unif, N_adap),
% 
% description:
%     generate a mesh triangulation of the earth, land and sea
% 
% arguments:
%   - N_unif  the number of uniformly distributed points across the flatland
%   - N_adap  the total number of points after increasing shore resolution (greater than N_unif)
% returns:
%   - pts     nodal points. (x,y)-coordinates for point i given in row i
%   - tri     elements. Index to the three corners of element i given in row i
%   - land    boolean vector stating if each point is on land or water

% author: Kjetil A. Johannessen
% last edit: November 2013


N         = N_unif;
stopN     = N_adap;
areaRatio = 2;
disp 'reading input'
if ~(exist('water')),
	water  = imread('water_16k.png');
end

[h, w] = size(water);
n1 = floor(sqrt(w/h*N));
n2 = floor(sqrt(h/w*N));
[x y] = meshgrid(1:n1, 1:n2);
x = x(:);
y = y(:);
pts = [x, y];

keepRunning = 1;

while keepRunning,
	[N, dontcare] = size(pts);
	N

	disp 'generating mesh'

	% final results for printing:
	tri = delaunay(x(:), y(:));
	u        = zeros(N,1);   % land mass 
	lat      = zeros(N,1);
	long     = zeros(N,1);

	disp 'image lookup';
	for i=1:N,
		lat(i)  = y(i);
		long(i) = x(i);

		[h, w] = size(water);
		x1 = x(i)/max(x)          * (w-1) + 1;
		y1 = (max(y)-y(i))/max(y) * (h-1) + 1;
		newU = water(round(y1),round(x1));

		u(i) = 255 * (newU ~= 255);
	end

	if N < stopN,
		disp 'refining mesh'
		[nTri dontcare] = size(tri);
		newHalf = zeros(nTri*3,2);
		nNew = 0;
		allArea = zeros(nTri,1);
		for j=1:nTri,
			p1 = [pts(tri(j,1),:),0];
			p2 = [pts(tri(j,2),:),0];
			p3 = [pts(tri(j,3),:),0];
			allArea(j) = norm(cross(p3-p1, p3-p2))/2.0;
		end
		maxArea = max(allArea);
		minArea = min(allArea);
		for j=1:nTri,
			if(abs(u(tri(j,1)) - u(tri(j,2))) > 100 || ...
			   abs(u(tri(j,1)) - u(tri(j,3))) > 100 || ...
			   abs(u(tri(j,2)) - u(tri(j,3))) > 100 || ...
			   allArea(j)/minArea > areaRatio),
				newHalf(nNew+1:nNew+3,:) = [tri(j,:)', tri(j,[2,3,1])'];
				nNew = nNew + 3;
			end
		end
		for j=1:nNew,
			newHalf(j,:) = sort(newHalf(j,:));
		end

		disp 'sorting unique new points'
		newHalf = sortrows(newHalf(1:nNew,:));
		newP    = zeros(nNew,2);
		nUnique = 0;
		
		for j=1:nNew,
			if j>1 && isequal(newHalf(j-1,:), newHalf(j,:)),
				continue;
			end
			nUnique = nUnique + 1;
			newP(nUnique,:) = (pts(newHalf(j,1),:)+pts(newHalf(j,2),:)) / 2.0;
		end

		newP = newP(1:nUnique,:);
		pts = [pts; newP];
		x   = pts(:,1);
		y   = pts(:,2);

		if nNew == 0,
			keepRunning = 0;
		end
	else
		keepRunning = 0;
	end
end

[nTri dontcare] = size(tri);
[nPts dontcare] = size(u);
disp 'Removing 1pixel lakes'
newU    = zeros(size(u));
evCount = zeros(size(u));
for i=1:nTri,
	t = tri(i,:);
	newU(t) = newU(t) + sum(u(t)==255);
	evCount(t) = evCount(t) + 1;
end
newU = newU ./ evCount / 2;
for i=1:nPts,
	if(newU(i) == 1),
		u(i) = 255;
	end
end

% flip everything around to make it the right way
pts = pts/max(max(pts));
x   = pts(:,1);
y   = pts(:,2);

c = u / 255.0 * 7.0 + (x+y);


% dump output arguments
land = u==255;


%%% - Uncomment below to plot results in MATLAB  -  %%%

% trisurf(tri, pts(:,1), pts(:,2), c);
% caxis([0, 15]);
% shading interp; hold on;
% trimesh(tri, pts(:,1), pts(:,2), c, 'EdgeColor', 'Black', 'FaceColor', 'None');
% view(2);
% figure; trimesh(tri,x,y);


