function [pts, tri, land, lat, long] = getEarthSphere(N_unif, N_adap),
% function [pts, tri, land, lat, long] = getEarthSphere(N_unif, N_adap),
% 
% description:
%     generate a mesh triangulation of the earth, land and sea
% 
% arguments:
%   - N_unif  the number of uniformly distributed points across the globe
%   - N_adap  the total number of points after increasing shore resolution (greater than N_unif)
% returns:
%   - pts     nodal points. (x,y,z)-coordinates for point i given in row i (normalized to r=1)
%   - tri     elements. Index to the three corners of element i given in row i
%   - land    boolean vector stating if each point is on land or water
%   - lat     latitude for each point
%   - long    lonlitude for each point. Can be used to provide meta-information

% author: Kjetil A. Johannessen
% last edit: November 2013

startN = N_unif;
stopN  = N_adap;
R = 1.0;  % radius of water levels
L = 1.0;  % radius of land levels
N = startN;
areaRatio = 100;
nSmooth = 5;

disp 'reading input'
if ~(exist('water')),
	water  = imread('water_16k.png');
end

pts = shell(N, R);

keepRunning = 1;

while keepRunning,
	[N, dontcare] = size(pts);
	N

	disp 'generating mesh'

	pts = [pts; 0,0,0];
	tetr = delaunay(pts(:,1), pts(:,2), pts(:,3));
	rep = TriRep(tetr, pts);

	% final results for printing:
	tri = freeBoundary(rep); % element matrix, triangulitation
	pts = pts(1:end-1, :);   % (x,y,z)-coordinates
	u = zeros(N,1);          % land mass 
	allPhi   = zeros(N,1);
	allTheta = zeros(N,1);

	disp 'image lookup';
	for i=1:N,
		r           = norm(pts(i,1:2));
		theta       = atan2(pts(i,2), pts(i,1));
		phi         = atan2(r, pts(i,3));
		allPhi(i)   = phi;
		allTheta(i) = theta;

		[h, w] = size(water);
		x1 = (theta+pi)/2/pi * (w-1) + 1;
		y1 = (phi)/pi        * (h-1) + 1;
		newU = water(round(y1),round(x1));

		u(i) = 255 * (newU ~= 255);
	end

	if N < stopN,
		disp 'refining mesh'
		[nTri dontcare] = size(tri);
		newHalf = zeros(nTri*3,2);
		nNew = 0;
		allArea = zeros(nTri,1);
		for j=1:nTri,
			p1 = pts(tri(j,1),:);
			p2 = pts(tri(j,2),:);
			p3 = pts(tri(j,3),:);
			allArea(j) = norm(cross(p3-p1, p3-p2))/2.0;
		end
		maxArea = max(allArea);
		minArea = min(allArea);
		for j=1:nTri,
			if(abs(u(tri(j,1)) - u(tri(j,2))) > 100 || ...
			   abs(u(tri(j,1)) - u(tri(j,3))) > 100 || ...
			   abs(u(tri(j,2)) - u(tri(j,3))) > 100 || ...
			   allArea(j)/minArea > areaRatio),
				newHalf(nNew+1:nNew+3,:) = [tri(j,:)', tri(j,[2,3,1])'];
				nNew = nNew + 3;
			end
		end
		for j=1:nNew,
			newHalf(j,:) = sort(newHalf(j,:));
		end

		disp 'sorting unique new points'
		newHalf = sortrows(newHalf(1:nNew,:));
		newP    = zeros(nNew,3);
		nUnique = 0;
		
		for j=1:nNew,
			if j>1 && isequal(newHalf(j-1,:), newHalf(j,:)),
				continue;
			end
			nUnique = nUnique + 1;
			newP(nUnique,:) = (pts(newHalf(j,1),:)+pts(newHalf(j,2),:)) / 2.0;
		end

		newP = newP(1:nUnique,:);
		for j=1:nUnique,
			newP(j,:) = newP(j,:) / norm(newP(j,:)) * R;
		end
		pts = [pts; newP];

		if nNew == 0,
			keepRunning = 0;
		end
	else
		keepRunning = 0;
	end
end

[nTri dontcare] = size(tri);
[nPts dontcare] = size(u);
disp 'Removing 1pixel lakes'
newU    = zeros(size(u));
evCount = zeros(size(u));
for i=1:nTri,
	t = tri(i,:);
	newU(t) = newU(t) + sum(u(t)==255);
	evCount(t) = evCount(t) + 1;
end
newU = newU ./ evCount / 2;
for i=1:nPts,
	if(newU(i) == 1),
		u(i) = 255;
	end
end


disp 'setting height curves'
[N, dontcare] = size(pts);
for i=1:N,
	if(u(i) ~= 0),
		pts(i,:) = pts(i,:) / R * L;
	end
end

c = u / 255.0 * 15.0 + (allPhi+allTheta);


% dump output arguments
lat  = (pi/2-phi)/pi*2 * 90.0;
long = theta/pi * 180.0;
land = u==255;



%%% - Uncomment below to plot results in MATLAB  -  %%%

trisurf(tri, pts(:,1), pts(:,2), pts(:,3), c);
caxis([-pi, 32]);
shading interp; hold on;
trimesh(tri, pts(:,1), pts(:,2), pts(:,3), 'EdgeColor', 'Black', 'FaceColor', 'None');

